//
//  UClass.h
//  usctp
//
//  Created by Andreas Fink on 10.10.20.
//  Copyright © 2020 Andreas Fink. All rights reserved.
//

#ifndef UCLASS_H
#define UCLASS_H 1

#include <stdint.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "UMutex.h"
struct UObject;
typedef enum UObjectComparisonResult
{
    UObjectOrderAscending = 1,
    UObjectOrderDescending = -1,
    UObjectOrderEqual = 0,
} UObjectComparisonResult;

struct UString;
typedef struct UClass
{
    const char                  *_name;
    struct UClass               *_superclass;
    size_t                      _size;
    int                         _version;
    int                         _globalAllocCounter;
    UMutex                      *_mutex;
    void                        (*_dealloc_func)(struct UObject *uc);
    int                         (*_isEqualFunction)(struct UObject *a, struct UObject *b); /* returns 1 if the two objects are considered equal (same content) */
    UObjectComparisonResult     (*_comparisonFunction)(struct UObject *a, struct UObject *b);
    struct UString             *(*_descriptionFunction)(struct UObject *a);
} UClass;


#define UO_CLASS_DEFINITION_SUPER_DEALLOC(NAME,SUPER,DEALLOC)   \
extern UClass SUPER##_class;   \
UClass NAME##_class =           \
{                               \
    #NAME,                      \
    &SUPER##_class,             \
    sizeof(NAME),               \
    1,                          \
    0,                          \
    NULL,                       \
    DEALLOC,                    \
    NULL,                       \
    NULL,                       \
    NULL                        \
}

#define UO_CLASS_DEFINITION_LONG(NAME,SUPER,DEALLOC,EQUAL,COMPARE,DESCRIPTION)  \
extern UClass SUPER##_class;   \
UClass NAME##_class =           \
{                               \
    #NAME,                      \
    &SUPER##_class,             \
    sizeof(NAME),               \
    1,                          \
    0,                          \
    NULL,                       \
    DEALLOC,                    \
    EQUAL,                      \
    COMPARE,                    \
    DESCRIPTION                 \
}


#endif /* UCLASS_H */

