//
//  udata.h
//  usctp
//
//  Created by Andreas Fink on 10.10.20.
//  Copyright © 2020 Andreas Fink. All rights reserved.
//

/* udata
 *
 * a container for arbitrary data
 *
 */

#if !defined(UDATA_H)
#define UDATA_H   1

#include "UObject.h"

typedef struct UData
{
    UObject     _u;
    ssize_t     _length;
    uint8_t     *_bytes;
} UData;

UData *udata_alloc(void);
UData *udata_init(UData *ud);
UData *udata_init_with_bytes(UData *ud,void *bytes, size_t len);
UData *udata_append_udata(UData *u, UData *u2);
UData *udata_append_bytes(UData *u, const uint8_t *bytes, size_t len);
UData *udata_subsection(UData *u, size_t start, size_t len);
UData *udata_head(UData *u, size_t pos);
UData *udata_tail(UData *u, size_t pos);
UData *udata_append_string(UData *u, const char *str);
UData *udata_append_char(UData *u, const char c);
UData *udata_append_nint64(UData *u, uint64_t i);
UData *udata_append_nint32(UData *u, uint32_t i);
UData *udata_append_nint16(UData *u, uint16_t i);
UData *udata_append_nint8(UData *u, uint8_t i);
uint64_t udata_grab_hint64(UData *u, size_t pos);
uint32_t udata_grab_hint32(UData *u, size_t pos);
uint16_t udata_grab_hint16(UData *u, size_t pos);
uint8_t udata_grab_hint8(UData *u, size_t pos);
char udata_grab_char(UData *u, size_t pos);
size_t udata_length(UData *u);
UData *udata_copy(UData *u);
int udata_interation_append(UObject *o, void *private_data);

#endif /* UDATA_H */

