//
//  ulist.h
//  usctp
//
//  Created by Andreas Fink on 10.10.20.
//  Copyright © 2020 Andreas Fink. All rights reserved.
//

/* ulist
 *
 * a container for a list of arbitrary objects
 *
 */

#if !defined(ULIST_H)
#define ULIST_H   1

#include "UObject.h"

#define ULIST_BLOCK_COUNT    64

typedef struct UList
{
    UObject         _u;
    int             _block_count;
    int             _used_total_count;
    struct UList    *_next;
    struct UList    *_previous;
    void            *(_list_entries[ULIST_BLOCK_COUNT]);
} UList;

typedef int (*ulist_iterate_func_ptr)(UObject *o, void *private_data); /* function should return 0 if it matches */

UList *ulist_alloc(void);
UList *ulist_init(UList *e);

void ulist_add(UList *list,UObject *o);
void ulist_remove(UList *list,UObject *o);

UObject *ulist_get(UList *e, int index); /* does not remove it */
UObject *ulist_remove_and_return_retained(UList *e, int index);
void ulist_delete(UList *e, int index);

int ulist_iterate(UList *e, ulist_iterate_func_ptr func, void *private_data); /* returns -1 if not found or index of found entry */
size_t ulist_count(UList *u);

#endif /* ULIST_H */

