//
//  uobject.h
//  usctp
//
//  Created by Andreas Fink on 10.10.20.
//  Copyright © 2020 Andreas Fink. All rights reserved.
//

/* uobject
 *
 * our super class c-object
 * all objects derived from it should have this structure as a header
 *
 */

#if !defined(UOBJECT_H)
#define UOBJECT_H   1

#include "UClass.h"
#include "UMutex.h"

typedef enum UObject_status
{
    UObject_status_allocated = 1,
    UObject_status_initialized = 2,
    UObject_status_freed = 3,
} UObject_status;

struct UString;

typedef struct UObject
{
    UClass          *_classpointer;
    size_t          _total_size;
    int32_t         _usage_counter;
    UObject_status  _status;
    UMutex          _mutex;
} UObject;

typedef void    (*uodealloc_func_ptr)(UObject *u);

UObject *uobject_alloc(UClass *classpointer);
UObject *uobject_init(UObject *uc);
UObject *uobject_retain(UObject *uc);
void    uobject_release(UObject *uc);
void    uobject_lock(UObject *uc);
void    uobject_unlock(UObject *uc);
int     uobject_trylock(UObject *uc); /* returns 0 if successful */
void uobject_assert(UObject *u); /* throws assertion if pointer is null or points to a deallocated object or with retain counter < 1 */
struct UString *uobject_description(UObject *o);



#endif /* UOBJECT_H */

#define U_ASSERT(a)         uobject_assert((UObject *)a)
#define U_RETAIN(a)         (a ? uobject_retain((UObject *)a) : NULL)
#define U_RELEASE(a)        (a ? uobject_release((UObject *)a) : NULL)
#define U_RELEASE_ZERO(a)   { U_RELEASE(a); a=NULL; }
#define U_LOCK(a)           (a ? uobject_lock((UObject *)a)   : NULL)
#define U_UNLOCK(a)         (a ? uobject_unlock((UObject *)a) : NULL)
#define UO_ALLOC(type)    (type *)uobject_alloc(&type##_class)

