//
//  UPacket.h
//  usctp
//
//  Created by Andreas Fink on 12.10.20.
//  Copyright © 2020 Andreas Fink. All rights reserved.
//

#ifndef UPACKET_H
#define UPACKET_H 1

#include "UObject.h"
#include "UData.h"
#include "UList.h"
#include "UIP_Address.h"
#include "USctp_Types.h"
#include "UInterface.h"

typedef struct UPacket
{
    UObject         _u;
    UInterface      *_interface;
    UData           *_ethernet_frame;
    UData           *_ethernet_src;
    UData           *_ethernet_dst;
    UData           *_ip_frame;
    UIP_Address     *_ip_src;
    UIP_Address     *_ip_dst;
    int             _ip_protocol;
    int             _source_port;
    int             _destination_port;
    UData           *_ip_payload;
    UList           *_sctp_chunks;
    const char      *_error;
} UPacket;


UPacket *upacket_alloc(void);
UPacket *upacket_init(UPacket *u);

void upacket_parse_icmp(UPacket *u);
void upacket_parse_tcp(UPacket *u);
void upacket_parse_udp(UPacket *u);
void upacket_parse_sctp(UPacket *u);

void upacket_process_sctp_data(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_init(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_init_ack(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_sack(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_heartbeat(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_heartbeat_ack(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_abort(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_shutdown(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_shutdown_ack(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_error(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_cookie_echo(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_cookie_ack(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_ecne(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_cwr(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void upacket_process_sctp_shutdown_complete(UPacket *u, usctp_chunk_header *chunk,size_t maxlen);





#endif /* UPACKET_H */
