//
//  USctp_Instance.h
//  usctp
//
//  Created by Andreas Fink on 08.11.2019.
//  Copyright © 2019 Andreas Fink. All rights reserved.
//

#if !defined(USCTP_INSTANCE_H)
#define USCTP_INSTANCE_H 1

#include "USctp_Types.h"
#include "UPacket.h"
#include "USctp_Assoc.h"
#include "UList.h"
#include "UTimer.h"

struct UAPI_Connection;

typedef struct USctp_Instance
{
    UObject         _u;
    UList           *_assocs;
    UList           *_apiConnections;
    /* timer values */
    float           _rto_initial;
    float           _rto_min;
    float           _rto_max;
    int             _max_burst;
    float           _rto_alpha;
    float           _rto_beta;
    float           _valid_cookie_life;
    int             _association_max_retrans;
    int             _path_max_retrans;
    int             _max_init_retransmits;
    float           _hb_interval;
    int             _hb_ma_burst;
} USctp_Instance;

USctp_Instance *usctp_instance_alloc(void);
USctp_Instance *usctp_instance_init(USctp_Instance *u);

void usctp_instance_add_assoc(USctp_Instance *s, USctp_Assoc *a);
void usctp_instance_remove_assoc(USctp_Instance *s, USctp_Assoc *a);
void usctp_instance_add_apiconnection(USctp_Instance *s, struct UAPI_Connection *a);
void usctp_instance_remove_apiconnection(USctp_Instance *s, struct UAPI_Connection *a);

void usctp_instance_process_sctp_data(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_init(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_init_ack(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_sack(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_heartbeat(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_heartbeat_ack(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_abort(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_shutdown(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_shutdown_ack(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_error(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_cookie_echo(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_cookie_ack(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_ecne(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_cwr(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);
void usctp_instance_process_sctp_shutdown_complete(USctp_Instance *s, UPacket *u, usctp_chunk_header *chunk,size_t maxlen);

#endif /* USCTP_INSTANCE_H  */
