//
//  USctp_Types.h
//  usctp
//
//  Created by Andreas Fink on 08.11.2019.
//  Copyright © 2019 Andreas Fink. All rights reserved.
//

#if !defined(USCTP_TYPES_H)
#define USCTP_TYPES_H   1

#include <stdint.h>
#include <arpa/inet.h>

/* network byte order (NBO) integers */

typedef uint8_t         nbo_uint8_t;
typedef uint16_t        nbo_uint16_t;
typedef uint32_t        nbo_uint32_t;
typedef uint64_t        nbo_uint64_t;

#define SET_NBO_UINT8(a)    (a)
#define GET_NBO_UINT8(a)    (a)

#define SET_NBO_UINT16(a)   (htons(a))
#define GET_NBO_UINT16(a)   (ntohs(a))

#define SET_NBO_UINT32(a)   (htonl(a))
#define GET_NBO_UINT32(a)   (ntohl(a))

#define SET_NBO_UINT64(a)   (htonll(a))
#define GET_NBO_UINT64(a)   (ntohll(a))


#if !defined(__Userspace_os_Windows)
#define USCTP_PACKED __attribute__((packed))
#else
#pragma pack (push, 1)
#define USCTP_PACKED
#endif


struct usctp_common_header
{
    nbo_uint16_t    source_port;
    nbo_uint16_t    destination_port;
    nbo_uint32_t    verification_tag;
    nbo_uint32_t    checksum;
} USCTP_PACKED;

typedef struct usctp_common_header usctp_common_header;


struct usctp_chunk_header
{
    nbo_uint8_t     chunk_type;
    nbo_uint8_t     chunk_flags;
    nbo_uint16_t    chunk_length;
} USCTP_PACKED;

typedef struct usctp_chunk_header usctp_chunk_header;


struct usctp_chunk_data_header
{
    usctp_chunk_header  header;
    nbo_uint32_t        transmission_sequence_number;
    nbo_uint16_t        stream_identifier;
    nbo_uint16_t        stream_sequence_number;
    nbo_uint32_t        payload_protocol_identifier;
    /* followed by data */
} USCTP_PACKED;

struct usctp_parameter_header
{
    nbo_uint16_t    parameter_type;
    nbo_uint16_t    parameter_len;
} USCTP_PACKED;

typedef struct usctp_parameter_header usctp_parameter_header;


typedef struct sctp_chunk_data_header sctp_chunk_data_header;


#define SCTP_DATA_FLAG_UNORDERED   0x04
#define SCTP_DATA_FLAG_BEGINNING   0x02
#define SCTP_DATA_FLAG_ENDING      0x01


typedef enum usctp_chunk_type
{
    USCTP_CHUNK_DATA = 0,
    USCTP_CHUNK_INIT = 1,
    USCTP_CHUNK_INIT_ACK = 2,
    USCTP_CHUNK_SACK = 3,
    USCTP_CHUNK_HEARTBEAT = 4,
    USCTP_CHUNK_HEARTBEAT_ACK = 5,
    USCTP_CHUNK_ABORT = 6,
    USCTP_CHUNK_SHUTDOWN = 7,
    USCTP_CHUNK_SHUTDOWN_ACK = 8,
    USCTP_CHUNK_ERROR = 9,
    USCTP_CHUNK_COOKIE_ECHO = 10,
    USCTP_CHUNK_COOKIE_ACK = 11,
    USCTP_CHUNK_ECNE = 12, /* reserved */
    USCTP_CHUNK_CWR = 13,  /* reserved */
    USCTP_CHUNK_SHUTDOWN_COMPLETE = 14,
} usctp_chunk_type;



#endif /* USCTP_TYPES_H  */
