//
//  UTcp_Socket.h
//  usctp
//
//  Created by Andreas Fink on 16.10.20.
//  Copyright © 2020 Andreas Fink. All rights reserved.
//
//

#if !defined(UTCP_SOCKET_H)
#define UTCP_SOCKET_H 1

#include "UObject.h"
#include "UIP_Address.h"
#include "UData.h"

struct UTcp_Socket;

typedef enum UTCP_SOCKET_OPTION
{
    UTCP_SOCKET_IPV4_ONLY   = 0x01,
    UTCP_SOCKET_IPV6_ONLY   = 0x02,
    UTCP_SOCKET_IPV46       = (UTCP_SOCKET_IPV6_ONLY | UTCP_SOCKET_IPV4_ONLY),
    UTCP_SOCKET_REUSE_ADDR  = 0x10,
    UTCP_SOCKET_REUSE_PORT  = 0x20,
    UTCP_SOCKET_NONBLOCKING = 0x40,
    UTCP_SOCKET_LINGER_3SEC = 0x80, /* sets linger to 3 seconds */
    UTCP_SOCKET_LINGER_ZERO = 0x100, /* sets linger to 0 seconds */
    UTCP_SOCKET_KEEPALIVE   = 0x200,
} UTCP_SOCKET_OPTION;

typedef struct UTcp_Socket
{
    UObject         _u;
    int             _isock;
    UIP_Address     *_bound_ip;
    UIP_Address     *_remote_ip;
    int             _lastError;
    int             _options;
} UTcp_Socket;

UTcp_Socket *utcp_socket_alloc(void);
UTcp_Socket *utcp_socket_init(UTcp_Socket *u);
UTcp_Socket *utcp_socket_create_with_option(UTcp_Socket *u,int option);


int utcp_socket_bind(UTcp_Socket *u, UIP_Address *ip, int port);
int utcp_socket_conect(UTcp_Socket *u, UIP_Address *ip, int port);
int utcp_socket_listen(UTcp_Socket *u, int backlog);
int utcp_socket_accept(UTcp_Socket *u, UTcp_Socket **createdConnection);
ssize_t utcp_socket_write_data(UTcp_Socket *u, UData *data);
ssize_t utcp_socket_write_bytes(UTcp_Socket *u, const uint8_t *bytes, size_t len);
ssize_t utcp_socket_read_data(UTcp_Socket *u, UData **data,size_t max);
ssize_t utcp_socket_read_skip(UTcp_Socket *u, size_t max);
int utcp_socket_close(UTcp_Socket *u);
int utcp_socket_force_close(UTcp_Socket *u);
int utcp_socket_poll(UTcp_Socket *u);

void utcp_socket_set_linger_time(UTcp_Socket *u, int linger_time);
void utcp_socket_set_linger_off(UTcp_Socket *u);
void utcp_socket_set_linger_on(UTcp_Socket *u);

void utcp_socket_set_receive_buffer_size(UTcp_Socket *u, int bufsize);
void utcp_socket_set_send_buffer_size(UTcp_Socket *u, int bufsize);
void utcp_socket_set_keepalive(UTcp_Socket *u, int keepalive);

#endif /* UTCP_SOCKET_H */
