//
//  UThread.h
//  usctp
//
//  Created by Andreas Fink on 14.10.20.
//  Copyright © 2020 Andreas Fink. All rights reserved.
//


#if !defined(UTHREAD_H)
#define UTHREAD_H 1

#include "UObject.h"
#include <pthread.h>

struct UThread;

typedef int (*uthread_start_fptr)(struct UThread *o, void *private_data);
typedef int (*uthread_stop_fptr)(struct UThread *o, void *private_data);
typedef int (*uthread_run_fptr)(struct UThread *o, void *private_data);
typedef int (*uthread_first_fptr)(struct UThread *o, void *private_data);
typedef int (*uthread_last_fptr)(struct UThread *o, void *private_data);

typedef struct UThread
{
    UObject             _u;
    uthread_start_fptr  _startFunc;
    uthread_stop_fptr   _stopFunc;
    uthread_run_fptr    _runFunc;
    uthread_first_fptr  _firstFunc;
    uthread_last_fptr   _lastFunc;
    int                 _starting; /* before thread started, this is set to 1. once thread has taken over argument, this is set to 0 */
    int                 _started; /* once thread has taken over argument*/
    int                 _mustRun; /* 1 after firstRun, while running procedure is looping */
    int                 _stopping; /* after main loop ended and last is being run */
    int                 _stopped;  /* when thread ended */
    int                 _doLoop; /* if the main routine shoudl be run continuously */
    pthread_t           _thread;
    void                *_arg;
} UThread;

UThread *uthread_alloc(void);
UThread *uthread_init(UThread *u);
UThread *uthread_init_with_run(UThread *u, int (*runFunc)(UThread *u, void *data));

int uthread_start(UThread *u, void *data);
int uthread_stop(UThread *u, void *data);
int uthread_run(UThread *u, void *data);
int uthread_first(UThread *u, void *data);
int uthread_last(UThread *u, void *data);

int uthread_default_start(UThread *u, void *data);
int uthread_default_stop(UThread *u, void *data);
int uthread_default_run(UThread *u, void *data);
int uthread_default_first(UThread *u, void *data);
int uthread_default_last(UThread *u, void *data);


#endif /* UTHREAD_H */
